# -*- encoding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>

Copyright (C) 2014-2021 OceanDataLab

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from __future__ import with_statement
import logging
import struct
try:
    import simplejson as json
except ImportError:
    import json
from webob import Response
from collections import defaultdict
import syntool_metadata
import syntool_metadata.db
import syntool_metadata.validation as validation

logger = logging.getLogger(__name__)
DAY_IN_SECONDS = 24 * 60 * 60


def get_version(environ, *args, **kwargs):
    """"""
    return syntool_metadata.__version__


def get_coverage(environ, *args, **kwargs):
    """ """
    required = ('webservice_cfg',)
    validation.require_params(kwargs, required)

    extent = kwargs.get('extent', None)
    if extent is not None and 1 > len(extent):
        extent = None

    products_filter = kwargs.get('products', None)
    if products_filter is not None:
        if 1 > len(products_filter):
            products_filter = None
        else:
            products_filter = products_filter.split(',')

    patterns = kwargs.get('patterns', None)
    if patterns is not None and 0 < len(patterns):
        patterns = patterns.split(',')
    else:
        patterns = None

    endianess = '<'  # Little Endian by default
    if kwargs.get('endianess') == 'big':
        endianess = '>'

    storage_type = kwargs['webservice_cfg']['storage_type']
    with storage_type.get_session() as storage:
        result = syntool_metadata.db.get_coverage(storage, extent,
                                                  products_filter, patterns)

    serialized = struct.pack('{}{}L'.format(endianess, len(result)), *result)
    return Response(body=serialized, content_type='application/octet-stream')


def get_datasets(environ, *args, **kwargs):
    """ """
    required = ('webservice_cfg', 'minDate', 'maxDate',)
    validation.require_params(kwargs, required)

    extent = kwargs.get('extent', None)
    if extent is not None and 1 > len(extent):
        extent = None

    products_filter = kwargs.get('products', None)
    if products_filter is not None:
        if 1 > len(products_filter):
            products_filter = None
        else:
            products_filter = products_filter.split(',')

    patterns = kwargs.get('patterns', None)
    if patterns is not None and 0 < len(patterns):
        patterns = patterns.split(',')
        patterns_by_product = defaultdict(list)
        for product, pattern in zip(products_filter, patterns):
            patterns_by_product[product].append(pattern)
    else:
        patterns_by_product = None

    min_date = validation.get_date_param(kwargs, 'minDate')
    max_date = validation.get_date_param(kwargs, 'maxDate')
    dates_only = 'dates_only' in kwargs

    storage_type = kwargs['webservice_cfg']['storage_type']
    ingested_dir = kwargs['webservice_cfg']['ingested_dir']
    results_limit = kwargs['webservice_cfg']['results_limit']
    with storage_type.get_session() as storage:
        datasets = syntool_metadata.db.get_datasets(storage, 'data/ingested',
                                                    ingested_dir, min_date,
                                                    max_date, extent,
                                                    products_filter,
                                                    patterns_by_product,
                                                    dates_only,
                                                    results_limit)

    if 'callback' in kwargs:
        return '{}({})'.format(kwargs['callback'], json.dumps(datasets))
    else:
        return datasets


def get_nearest(environ, *args, **kwargs):
    """ """
    required = ('webservice_cfg', 'date')
    validation.require_params(kwargs, required)

    extent = kwargs.get('extent', None)
    if extent is not None and 1 > len(extent):
        extent = None

    products = kwargs.get('products', None)
    if products is not None:
        if 1 > len(products):
            products = None
        else:
            products = products.split(',')

    patterns = kwargs.get('patterns', None)
    if patterns is not None and 0 < len(patterns):
        patterns = patterns.split(',')
    else:
        patterns = None

    current_date = validation.get_date_param(kwargs, 'date')
    in_past = 'past' in kwargs
    in_present = 'present' in kwargs
    in_future = 'future' in kwargs

    storage_type = kwargs['webservice_cfg']['storage_type']
    with storage_type.get_session() as storage:
        result = syntool_metadata.db.get_nearest(storage, extent, products,
                                                   patterns, current_date,
                                                   in_past, in_present,
                                                   in_future)
    return result


def list_products(environ, *args, **kwargs):
    """ """
    required = ('webservice_cfg',)
    validation.require_params(kwargs, required)

    storage_type = kwargs['webservice_cfg']['storage_type']
    with storage_type.get_session() as storage:
        results = syntool_metadata.db.list_products(storage)
    return '\n'.join(['{} : {}'.format(k, v) for k, v in results.items()])


def get_granule_info(environ, *args, **kwargs):
    """ """
    required = ('webservice_cfg', 'product_id', 'dataset_name')
    validation.require_params(kwargs, required)

    ingested_dir = kwargs['webservice_cfg']['ingested_dir']
    result = syntool_metadata.db.get_granule_info(ingested_dir,
                                                  kwargs['product_id'],
                                                  kwargs['dataset_name'])
    return json.dumps(result)
